/* $Id: drag.js 3815 2010-06-18 14:16:01Z denis $ */
/* DRAG AND DROP */

var dragLabel = null;

dragManager = {
  dragInProgress: false,
  dragLabel: null,
  draggedObject: null,
  dropTargetObject: null,
  dropPositionIndicator: null,
  dropPositionImages: {},

  //   
  dragEventX: 0,
  dragEventY: 0,
  //   dragLabel (          )
  dragLabelVisible: false,

  //   ID    netcat
  draggedInstance: {}, // { type: x, id: n }
  //    ,   
  droppedInstance: {},

  init: function() {
    this.dragLabel = createElement('DIV', {id: 'dragLabel', 'style.display': 'none'}, document.body);
    this.dropPositionIndicator = createElement('IMG', {
       id: 'dropPositionIndicator',
       'style.display': 'none',
       'style.position': 'absolute',
       'style.zIndex': 5000
      }, document.body);

    // preload drop position indicators
    this.dropPositionImages.arrowRight = new Image();
    this.dropPositionImages.arrowRight.src = ICON_PATH + 'drop_arrow_right.gif';

    this.dropPositionImages.arrowTop = new Image();
    this.dropPositionImages.arrowTop.src = ICON_PATH + 'drop_arrow_top.gif';

    this.dropPositionImages.line = new Image();
    this.dropPositionImages.line.src = ICON_PATH + 'drop_line.gif';

  }, // end of dragManager.init ------

  // init event listeners on drag start
  initHandlers: function() {
    dragManager.initHandlersInWindow(window);
    for (var i=0; i<window.frames.length; i++) {
      dragManager.initHandlersInWindow(window.frames[i]);
    }
  },  // end of initHandlers

  initHandlersInWindow: function(targetWindow) {
    // store frame coords
    if (targetWindow.frameElement) {
       targetWindow.frameOffset = getOffset(targetWindow.frameElement);
    }
    else {
       targetWindow.frameOffset = { left: 0, top: 0 };
    }

    // (1) onmousemove
    targetWindow.document.onmousemove = function(e) {
      if (targetWindow != top) targetWindow.scroller.scroll(e);

      if (targetWindow.event) { // IE
        e = targetWindow.event;
      }

      var x = e.clientX + targetWindow.frameOffset.left;
      var y = e.clientY + targetWindow.frameOffset.top;
      dragManager.labelMove(x, y);
    }

    targetWindow.document.onmouseout = function(e) {
      targetWindow.scroller.scrollStop(e);
    }

    // (2) onmouseup
    targetWindow.mouseUpEventId = bindEvent(targetWindow.document, 'mouseup', dragManager.dragEnd);
  },

  removeHandlers: function() {
    dragManager.removeHandlersInWindow(window);
    for (var i=0; i<window.frames.length; i++) {
      dragManager.removeHandlersInWindow(window.frames[i]);
    }
  },

  removeHandlersInWindow: function(targetWindow) {
    // (1) onmousemove
    targetWindow.document.onmousemove = null;
    targetWindow.document.onmouseout = null;
    if (targetWindow.scroller) targetWindow.scroller.scrollStop();
    // (2) onmouseup
    unbindEvent(targetWindow.mouseUpEventId);
  },


  idRegexp: /_?([a-z]+)(\d+)?\-([a-f\d]+)$/i, // class-123, message12-345

  //    ID  netcat  ID html-
  // ., "siteTree_sub-348") -> { type: 'sub', id: '348' }
  // "mainViewToolbar_subclass-223" -> { type: 'subclass', id: '223' }
  //   message -   typeNum
  // "message12-345 -> { type : 'message', typeNum: 12, id: 345 }
  getInstanceData: function(objectId) {
    var matches = objectId.match(dragManager.idRegexp);
    if (matches) {
      return { type: matches[1], typeNum: matches[2], id: matches[3] };
    }
    else return {};
  },

  /**
    * -  ,    droppable
    * - ,         
    * -  ,   
    *
    *   ** (applied)   (  bindEvent)
    */
  dropTargetMouseOver: function(e) {
    // this == ,    event (drop target)
    if (!dragManager.dragInProgress) { return false; }
    if (!this.acceptsDrop) { return false; }

    // Generic rule: cannot be dropped on itself nor to its descendants
    if (dragManager.draggedObject == this) { return false; }

    var parentObject = this.parentNode;
    while (parentObject) {
      if (parentObject == dragManager.draggedObject) { return false; }
      parentObject = parentObject.parentNode;
    }

    var droppedId = (this.id ? this.id : this.parentNode.id);
    dragManager.droppedInstance = dragManager.getInstanceData(droppedId);

    if (this.acceptsDrop()) {
      // save the object as current target for the drop
      dragManager.dropTargetObject = this;
      if (this.dropIndicator && dragManager.dropPositionImages[this.dropIndicator.name]) {
        var ind = dragManager.dropPositionIndicator;
        ind.src = dragManager.dropPositionImages[this.dropIndicator.name].src;
        var pos = getOffset(this, false, true),
            top = this.dropIndicator.top ? this.dropIndicator.top
                                         : this.offsetHeight + this.dropIndicator.bottom;

        ind.style.top = pos.top + top + 'px';
        ind.style.left = pos.left + this.dropIndicator.left + 'px';
        ind.style.display = '';
      }
    } // of accepts drop

    // stop event propagation
    if (e && e.stopPropagation) { e.stopPropagation(); }
    else {
      if (this.document.parentWindow.event) {
        this.document.parentWindow.event.cancelBubble = true;
      }
    }
  },

  dropTargetMouseOut: function(e) {
    if (!dragManager.dragInProgress) { return; }
    dragManager.dropTargetObject = null;
    dragManager.droppedInstance = {}
    dragManager.dropPositionIndicator.style.display = 'none';
  },


  labelSetHTML: function(html) {
    this.dragLabel.innerHTML = html;
  },

  //
  labelMove: function(x, y, frameId) {
    // show only if mouse moved already for more than 12px
    if (!this.dragLabelVisible) {
      if (Math.abs(x - this.dragEventX) > 12 || Math.abs(y - this.dragEventY) > 12) {
        this.dragLabelVisible = true;
        this.dragLabel.style.display = '';
      }
    }
    if (!this.dragLabelVisible) return;

    this.dragLabel.style.top = y + 10;
    this.dragLabel.style.left = x + 10;

  }, // end of dragManager.labelMove


  /**
    *   
    * @param {Object} -  ("", 
    *     ""  draggedObject)
    * @param {Object} ,   
    *   (  ,    handlerObject)
    */
  addDraggable: function(handlerObject, draggedObject) {
    handlerObject.ondragstart = dragManager.cancelEvent;
    if (!draggedObject) draggedObject = handlerObject;

    bindEvent(handlerObject, 'mousedown',
      function(e) {
        dragManager.dragStart(e ? e : window.event, draggedObject);
      }, true);
  },

  cancelEvent: function() { return false; },

  //  
  // IE: window.event    !
  dragStart: function(e, draggedObject) {
    // check if left button was pressed
    if ((e.stopPropagation && e.button != 0) ||    // DOM (Mozilla)
        (!e.stopPropagation && e.button != 1)      // IE
       ) { return; } // not a left mouse button

    dragManager.initHandlers();

    dragManager.draggedObject = draggedObject;
    dragManager.dragInProgress = true;

    var dragLabel = draggedObject.dragLabel;
    if (!dragLabel) {
      if (draggedObject.getAttribute('dragLabel')) {
        dragLabel = draggedObject.getAttribute('dragLabel');
      }
      else {
        dragLabel = draggedObject.innerHTML;
      }
    }

    dragManager.labelSetHTML(dragLabel);

    dragManager.draggedInstance = dragManager.getInstanceData(draggedObject.id);
    // store drag event coodrdinates
    var windowOffset = draggedObject.ownerDocument.defaultView ?
                         draggedObject.ownerDocument.defaultView.frameOffset :  // moz
                         draggedObject.ownerDocument.parentWindow.frameOffset;   // IE
    dragManager.dragEventX = e.clientX + windowOffset.left;
    dragManager.dragEventY = e.clientY + windowOffset.top;

    if (e.stopPropagation) { e.stopPropagation(); e.preventDefault(); }
    else if (e) { e.cancelBubble = true; }
  },

  //  
  dragEnd: function(e) {
    if (dragManager.dropTargetObject) { dragManager.dropTargetObject.dropHandler(); }
    dragManager.dragInProgress = false;
    dragManager.draggedObject = null;
    dragManager.dropTargetObject = null;
    dragManager.dragLabelVisible = false;
    dragManager.dragLabel.style.display = 'none';
    dragManager.dropPositionIndicator.style.display = 'none';
    dragManager.draggedInstance = {};
    dragManager.droppedInstance = {};
    dragManager.removeHandlers();
  },

/**
 *     
 * @param {Object} object ,     (drop)
 * @param {Function} acceptFn       object
 * @param {Function} onDrop ,      object
 * @param {Object} dropIndicator . dragManager.init() - position indicators.   
 *   { name, top|bottom, left }
 */
  addDroppable: function(object, acceptFn, onDropFn, dropIndicator) {
    object.acceptsDrop = acceptFn;
    object.dropHandler = onDropFn;
    object.dropIndicator = dropIndicator;

    bindEvent(object, 'mouseover', dragManager.dropTargetMouseOver);
    bindEvent(object, 'mouseout',  dragManager.dropTargetMouseOut);
  }
}



bindEvent(window, 'load', function() { dragManager.init(); });
